@@ -43,9 +43,11 @@ from gajim.common.helpers import warn_about_plain_connection
 from gajim.common.i18n import _
 from gajim.common.idle import IdleMonitorManager
 from gajim.common.idle import Monitor
+from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.message import build_message_stanza
 from gajim.common.structs import OutgoingMessage
 from gajim.common.util.http import create_http_session
+from gajim.common.util.standards import get_rfc5646_lang
 from gajim.common.util.status import get_idle_status_message
 from gajim.common.util.text import to_one_line
 
@@ -77,7 +79,6 @@ class Client(Observable, ClientModules):
         address = app.settings.get_account_setting(self._account, 'address')
         self._address = JID.from_string(address)
 
-        self._priority = 0
         self._connect_machine_calls = 0
         self.addressing_supported = False
 
@@ -132,10 +133,6 @@ class Client(Observable, ClientModules):
             return self._idle_status_message
         return self._status_message
 
-    @property
-    def priority(self) -> int:
-        return self._priority
-
     @property
     def certificate(self):
         return self._client.peer_certificate[0]
@@ -169,6 +166,7 @@ class Client(Observable, ClientModules):
 
         self._client = NBXMPPClient(log_context=self._account)
         self.connection = self._client
+        self._client.set_lang(get_rfc5646_lang())
         self._client.set_domain(self._address.domain)
         self._client.set_username(self._address.localpart)
         self._client.set_resource(get_resource(self._account))
@@ -278,12 +276,21 @@ class Client(Observable, ClientModules):
                                                  'resource',
                                                  'gajim.$rand')
             if error == 'system-shutdown':
+                account_label = app.settings.get_account_setting(
+                    self._account, "account_label"
+                )
+                account_text = _("Account: %s") % account_label
+                shutdown_text = text or _("The server was shut down.")
+
                 app.ged.raise_event(
-                    Notification(context_id="",
-                                 account=self._account,
-                                 type='server-shutdown',
-                                 title=_('Server Shutdown'),
-                                 text=text or _('The server was shut down.')))
+                    Notification(
+                        context_id="",
+                        account=self._account,
+                        type="server-shutdown",
+                        title=_("Server Shutdown"),
+                        text=f"{shutdown_text}\n{account_text}",
+                    )
+                )
 
         elif domain == StreamError.SASL:
             self._reconnect = False
@@ -358,6 +365,12 @@ class Client(Observable, ClientModules):
         own_jid = self.get_own_jid()
         return own_jid.bare_match(jid)
 
+    def get_own_contact(self) -> BareContact:
+        jid = self.get_own_jid()
+        contact = self.get_module("Contacts").get_contact(jid.bare)
+        assert isinstance(contact, BareContact)
+        return contact
+
     def get_own_jid(self) -> JID:
         '''
         Return the last full JID we received on a bind event.
@@ -441,9 +454,7 @@ class Client(Observable, ClientModules):
 
     def update_presence(self, include_muc: bool = True) -> None:
         status, message, idle = self.get_presence_state()
-        self._priority = app.get_priority(self._account, status)
         self.get_module('Presence').send_presence(
-            priority=self._priority,
             show=status,
             status=message,
             idle_time=idle)
@@ -468,10 +479,10 @@ class Client(Observable, ClientModules):
 
         self.get_module('Bookmarks').request_bookmarks()
         self.get_module('SoftwareVersion').set_enabled(True)
-        self.get_module('EntityTime').set_enabled(True)
         self.get_module('LastActivity').set_enabled(True)
         self.get_module('Annotations').request_annotations()
         self.get_module('Blocking').get_blocking_list()
+        self.get_module('VCard4').subscribe_to_node()
 
         if app.settings.get_account_setting(self._account, 'publish_tune'):
             self.get_module('UserTune').set_enabled(True)
