@@ -114,19 +114,26 @@ class AppSideBar(Gtk.Box, EventHelper):
             account = None
 
         texture = app.app.avatar_storage.get_account_button_texture(
-            account, AvatarSize.ACCOUNT_SIDE_BAR, self.get_scale_factor()
+            account,
+            AvatarSize.ACCOUNT_SIDE_BAR,
+            self.get_scale_factor(),
+            self._has_connectivity_issues(),
         )
         self._account_row.set_from_paintable(texture)
 
+    @staticmethod
+    def _has_connectivity_issues() -> bool:
+        for account in app.settings.get_active_accounts():
+            client = app.get_client(account)
+            if client.state.is_disconnected or client.state.is_reconnect_scheduled:
+                return True
+        return False
+
     def _on_account_clicked(
         self,
         _popover: AccountPopover,
         account: str,
     ) -> None:
-        if not account:
-            app.app.activate_action("accounts", GLib.Variant("s", ""))
-            return
-
         app.window.show_account_page(account)
         self._bottom_listbox.select_row(self._account_row)
 
@@ -147,8 +154,18 @@ class AppSideBar(Gtk.Box, EventHelper):
         current_button = gesture_click.get_current_button()
         if current_button == Gdk.BUTTON_PRIMARY:
             # Left click
+
             self._status_popover.popdown()
-            self._account_popover.popup()
+
+            accounts = app.settings.get_active_accounts()
+            if len(accounts) == 0:
+                return Gdk.EVENT_PROPAGATE
+
+            if len(accounts) == 1:
+                app.window.show_account_page(accounts[0])
+                self._bottom_listbox.select_row(self._account_row)
+            else:
+                self._account_popover.popup()
             return Gdk.EVENT_PROPAGATE
 
         if current_button == Gdk.BUTTON_SECONDARY:
@@ -267,12 +284,8 @@ class AccountPopover(Gtk.Popover):
         self._rows.clear()
 
     @Gtk.Template.Callback()
-    def _on_row_activated(
-        self, _listbox: Gtk.ListBox, row: AccountPopoverRow | Gtk.ListBoxRow
-    ) -> None:
-        account = ""
-        if isinstance(row, AccountPopoverRow):
-            account = row.get_account()
+    def _on_row_activated(self, _listbox: Gtk.ListBox, row: AccountPopoverRow) -> None:
+        account = row.get_account()
 
         self.emit("clicked", account)
         self.popdown()
@@ -297,10 +310,21 @@ class AccountPopoverRow(Gtk.ListBoxRow):
         label = app.settings.get_account_setting(account, "account_label")
         self._label.set_text(label)
 
+        client = app.get_client(account)
+        connectivity_issues = bool(
+            client.state.is_disconnected or client.state.is_reconnect_scheduled
+        )
         texture = app.app.avatar_storage.get_account_button_texture(
-            account, AvatarSize.ACCOUNT_SIDE_BAR, self.get_scale_factor()
+            account,
+            AvatarSize.ACCOUNT_SIDE_BAR,
+            self.get_scale_factor(),
+            connectivity_issues,
         )
         self._avatar.set_from_paintable(texture)
+        if connectivity_issues:
+            self._avatar.set_tooltip_text(_("There are connectivity issues"))
+        else:
+            self._avatar.set_tooltip_text("")
 
     def do_unroot(self) -> None:
         Gtk.ListBoxRow.do_unroot(self)
