@@ -7,8 +7,7 @@ from __future__ import annotations
 from typing import Any
 from typing import Literal
 
-from datetime import datetime
-from urllib.parse import urlparse
+import datetime as dt
 
 from gi.repository import Gdk
 from gi.repository import Gio
@@ -34,10 +33,7 @@ from gajim.common.storage.draft import DraftStorage
 from gajim.common.types import ChatContactT
 from gajim.common.util.muc import get_groupchat_name
 from gajim.common.util.muc import message_needs_highlight
-from gajim.common.util.preview import filename_from_uri
-from gajim.common.util.preview import format_geo_coords
-from gajim.common.util.preview import guess_simple_file_type
-from gajim.common.util.preview import split_geo_uri
+from gajim.common.util.preview import get_preview_data
 from gajim.common.util.user_strings import get_moderation_text
 from gajim.common.util.user_strings import get_retraction_text
 from gajim.common.util.user_strings import get_uf_relative_time
@@ -73,6 +69,7 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         self.workspace_id = workspace_id
         self.type = type_
         self.position = position
+        self._has_draft = False
 
         self._conversations_header = RowHeader(RowHeaderType.CONVERSATIONS)
         self._pinned_header = RowHeader(RowHeaderType.PINNED)
@@ -141,7 +138,9 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         if isinstance(self.contact, GroupchatContact) and not self.contact.can_notify():
             self._ui.unread_label.add_css_class("unread-counter-silent")
 
-        self._display_last_conversation_row()
+        draft = app.storage.drafts.get(self.contact)
+        if not self._set_draft_mode(draft):
+            self._display_last_conversation_row()
 
     def _display_last_conversation_row(self) -> None:
         message = app.storage.archive.get_last_conversation_row(
@@ -194,17 +193,21 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
             self.message_id = message.id
 
         if message.filetransfers:
-            self.set_message_text(_("File"), icon_name="text-x-generic-symbolic")
+            self.set_message_text(_("File"), icon_name="lucide-paperclip-symbolic")
             self.set_timestamp(message.timestamp)
 
         if message.call is not None:
-            self.set_message_text(_("Call"), icon_name="call-start-symbolic")
+            self.set_message_text(_("Call"), icon_name="lucide-phone-symbolic")
             self.set_timestamp(message.timestamp)
 
     @property
     def is_pinned(self) -> bool:
         return self._pinned
 
+    @property
+    def has_draft(self) -> bool:
+        return self._has_draft
+
     @property
     def unread_count(self) -> int:
         if (
@@ -259,31 +262,21 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         oob: list[mod.OOB] | None = None,
     ) -> None:
 
+        if self._has_draft:
+            return
+
         assert isinstance(
             self.contact, BareContact | GroupchatContact | GroupchatParticipant
         )
 
-        draft = app.storage.drafts.get(self.contact)
-        if draft is not None:
-            self._show_draft(draft)
-            return
-
-        self._ui.message_label.remove_css_class("draft")
-
         icon = None
         if icon_name is not None:
             icon = Gio.Icon.new_for_string(icon_name)
 
-        if app.preview_manager.is_previewable(text, oob or []):
-            scheme = urlparse(text).scheme
-            if scheme == "geo":
-                location = split_geo_uri(text)
-                icon = Gio.Icon.new_for_string("mark-location")
-                text = format_geo_coords(float(location.lat), float(location.lon))
-            else:
-                file_name = filename_from_uri(text)
-                icon, file_type = guess_simple_file_type(text)
-                text = f"{file_type} ({file_name})"
+        preview = get_preview_data(text, oob or [])
+        if preview is not None:
+            text = preview.text
+            icon = preview.icon
 
         text = GLib.markup_escape_text(text)
         if text.startswith("/me ") and nickname is not None:
@@ -294,6 +287,10 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         # Split by newline and display last line (or first, if last is newline)
         lines = text.split("\n")
         text = lines[-1] or lines[0]
+
+        if text.startswith("```"):
+            text = _("Code snippet")
+
         self._ui.message_label.set_markup(text)
 
         if icon is None:
@@ -312,7 +309,7 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
     def set_stanza_id(self, stanza_id: str | None) -> None:
         self.stanza_id = stanza_id
 
-    def set_timestamp(self, timestamp: datetime) -> None:
+    def set_timestamp(self, timestamp: dt.datetime) -> None:
         self.timestamp = timestamp.timestamp()
         self.update_time()
 
@@ -349,9 +346,8 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
     def update_time(self) -> None:
         if self.timestamp == 0:
             return
-        self._ui.timestamp_label.set_text(
-            get_uf_relative_time(datetime.fromtimestamp(self.timestamp))
-        )
+        utc_timestamp = dt.datetime.fromtimestamp(self.timestamp, dt.UTC)
+        self._ui.timestamp_label.set_text(get_uf_relative_time(utc_timestamp))
 
     def update_row_state(self) -> None:
         self.update_time()
@@ -421,17 +417,22 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         if contact != self.contact:
             return
 
-        self._show_draft(draft)
+        if not self._set_draft_mode(draft):
+            self._display_last_conversation_row()
+        self.changed()
+
+    def _set_draft_mode(self, draft: Draft | None) -> bool:
+        self._has_draft = draft is not None
 
-    def _show_draft(self, draft: Draft | None) -> None:
         if draft is None:
             self._ui.message_label.remove_css_class("draft")
-            self._display_last_conversation_row()
-            return
 
-        self.set_nick("")
-        self._ui.message_label.set_text(_("Draft: %s") % draft.text)
-        self._ui.message_label.add_css_class("draft")
+        else:
+            self.set_nick("")
+            self._ui.message_label.set_text(_("Draft: %s") % draft.text)
+            self._ui.message_label.add_css_class("draft")
+
+        return self._has_draft
 
     def _on_state_flags_changed(
         self, _row: ChatListRow, _flags: Gtk.StateFlags
@@ -458,6 +459,7 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         app.check_finalize(self)
 
     def _on_close_button_clicked(self, _button: Gtk.Button) -> None:
+        self.reset_unread()
         app.window.activate_action(
             "win.remove-chat", GLib.Variant("as", [self.account, str(self.jid)])
         )
@@ -571,14 +573,14 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
 
         assert isinstance(self.contact, GroupchatContact)
         if self.contact.is_joining:
-            self._ui.connection_icon.set_from_icon_name("feather-refresh-cw-symbolic")
+            self._ui.connection_icon.set_from_icon_name("lucide-refresh-cw-symbolic")
             self._ui.connection_icon.add_css_class("spin")
-            self._ui.connection_icon.add_css_class("dim-label")
+            self._ui.connection_icon.add_css_class("dimmed")
             self._ui.connection_icon.set_tooltip_text(_("Joining Group Chat…"))
             self._ui.connection_icon.set_visible(True)
         elif self.contact.is_not_joined or not self._client.state.is_available:
-            self._ui.connection_icon.set_from_icon_name("feather-zap-symbolic")
-            self._ui.connection_icon.add_css_class("warning-color")
+            self._ui.connection_icon.set_from_icon_name("lucide-zap-symbolic")
+            self._ui.connection_icon.add_css_class("warning")
             self._ui.connection_icon.set_tooltip_text(_("Not connected"))
             self._ui.connection_icon.set_visible(True)
 
@@ -589,9 +591,9 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         self._reset_connection_icon()
 
         if signal_name == "mam-sync-started":
-            self._ui.connection_icon.set_from_icon_name("feather-refresh-cw-symbolic")
+            self._ui.connection_icon.set_from_icon_name("lucide-refresh-cw-symbolic")
             self._ui.connection_icon.add_css_class("spin")
-            self._ui.connection_icon.add_css_class("info-color")
+            self._ui.connection_icon.add_css_class("accent")
             self._ui.connection_icon.set_tooltip_text(_("Fetching messages…"))
             self._ui.connection_icon.set_visible(True)
 
@@ -600,8 +602,8 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
     ) -> None:
 
         self._reset_connection_icon()
-        self._ui.connection_icon.set_from_icon_name("feather-zap-symbolic")
-        self._ui.connection_icon.add_css_class("error-color")
+        self._ui.connection_icon.set_from_icon_name("lucide-zap-symbolic")
+        self._ui.connection_icon.add_css_class("error")
         self._ui.connection_icon.set_tooltip_text(
             _("There has been an error while trying to fetch messages: %s") % error_text
         )
@@ -610,10 +612,10 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
     def _reset_connection_icon(self) -> None:
         self._ui.connection_icon.set_visible(False)
         self._ui.connection_icon.remove_css_class("spin")
-        self._ui.connection_icon.remove_css_class("dim-label")
-        self._ui.connection_icon.remove_css_class("info-color")
-        self._ui.connection_icon.remove_css_class("warning-color")
-        self._ui.connection_icon.remove_css_class("error-color")
+        self._ui.connection_icon.remove_css_class("dimmed")
+        self._ui.connection_icon.remove_css_class("accent")
+        self._ui.connection_icon.remove_css_class("warning")
+        self._ui.connection_icon.remove_css_class("error")
 
     def _on_muc_user_update(
         self, _contact: GroupchatParticipant, _signal_name: str, *args: Any
