@@ -22,10 +22,8 @@ from gajim.common.i18n import _
 from gajim.common.setting_values import BoolSettings
 
 # from gajim.common.util.av import AudioOutputManager
-from gajim.common.util.av import AudioInputManager
-
 # from gajim.common.util.av import VideoInputManager
-from gajim.common.util.uri import open_directory
+from gajim.common.util.av import AudioInputManager
 from gajim.common.util.version import package_version
 
 from gajim.gtk.alert import ConfirmationAlertDialog
@@ -33,11 +31,11 @@ from gajim.gtk.const import Setting
 from gajim.gtk.const import SettingKind
 from gajim.gtk.const import SettingType
 from gajim.gtk.plugins import Plugins
-from gajim.gtk.preview import PREVIEW_ACTIONS
 from gajim.gtk.settings import DropDownSetting
 from gajim.gtk.settings import GajimPreferencePage
 from gajim.gtk.settings import GajimPreferencesGroup
 from gajim.gtk.sidebar_switcher import SideBarMenuItem
+from gajim.gtk.util.misc import open_file
 from gajim.gtk.util.window import get_app_window
 from gajim.gtk.util.window import open_window
 
@@ -95,6 +93,7 @@ class WindowBehaviourGroup(GajimPreferencesGroup):
                 SettingType.CONFIG,
                 "show_header_bar",
                 desc=_("Show the window’s title bar"),
+                enabled_func=lambda: sys.platform != "win32",
             ),
         ]
 
@@ -276,7 +275,7 @@ class GroupChatsGroup(GajimPreferencesGroup):
             ),
             Setting(
                 SettingKind.SWITCH,
-                _("Show Subject"),
+                _("Show Subject on Join"),
                 SettingType.CONFIG,
                 "show_subject_on_join",
             ),
@@ -354,19 +353,14 @@ class FilePreviewGroup(GajimPreferencesGroup):
             26214400: "25 MiB",
         }
 
-        preview_actions = {}
-        for action, data in PREVIEW_ACTIONS.items():
-            if action == "download":
-                continue
-            preview_actions[action] = data[0]
-
         settings = [
             Setting(
-                SettingKind.SWITCH,
-                _("File Preview"),
+                SettingKind.DROPDOWN,
+                _("File Size Limit"),
                 SettingType.CONFIG,
-                "enable_file_preview",
-                desc=_("Show previews for files"),
+                "preview_max_file_size",
+                desc=_("Maximum file size for preview downloads"),
+                props={"data": sizes},
             ),
             Setting(
                 SettingKind.SPIN,
@@ -374,18 +368,8 @@ class FilePreviewGroup(GajimPreferencesGroup):
                 SettingType.CONFIG,
                 "preview_size",
                 desc=_("Size of preview images in pixels"),
-                bind="enable_file_preview",
                 props={"range_": (100, 1000, 1)},
             ),
-            Setting(
-                SettingKind.DROPDOWN,
-                _("File Size Limit"),
-                SettingType.CONFIG,
-                "preview_max_file_size",
-                desc=_("Maximum file size for preview downloads"),
-                bind="enable_file_preview",
-                props={"data": sizes},
-            ),
             Setting(
                 SettingKind.SWITCH,
                 _("Preview in Public Group Chats"),
@@ -395,7 +379,6 @@ class FilePreviewGroup(GajimPreferencesGroup):
                     "Show previews automatically in public "
                     "group chats (may disclose your data)"
                 ),
-                bind="enable_file_preview",
             ),
             Setting(
                 SettingKind.SWITCH,
@@ -403,16 +386,6 @@ class FilePreviewGroup(GajimPreferencesGroup):
                 SettingType.CONFIG,
                 "preview_allow_all_images",
                 desc=_("Show previews for any URLs containing images (may be unsafe)"),
-                bind="enable_file_preview",
-            ),
-            Setting(
-                SettingKind.DROPDOWN,
-                _("Left Click Action"),
-                SettingType.CONFIG,
-                "preview_leftclick_action",
-                desc=_("Action for left-clicking a preview"),
-                bind="enable_file_preview",
-                props={"data": preview_actions},
             ),
             Setting(
                 SettingKind.SWITCH,
@@ -423,7 +396,6 @@ class FilePreviewGroup(GajimPreferencesGroup):
                     "Whether to check for a valid certificate before "
                     "downloading (not safe to disable)"
                 ),
-                bind="enable_file_preview",
             ),
         ]
 
@@ -442,40 +414,27 @@ class VisualNotificationsGroup(GajimPreferencesGroup):
         settings = [
             Setting(
                 SettingKind.SWITCH,
-                _("Notification Area Icon"),
+                _("System Tray Icon"),
                 SettingType.CONFIG,
                 "show_trayicon",
             ),
             Setting(
-                SettingKind.SUBPAGE,
-                _("Show Notifications"),
+                SettingKind.SWITCH,
+                _("System Tray Notifications"),
                 SettingType.CONFIG,
-                "show_notifications",
-                props={"subpage": "notifications"},
+                "trayicon_notification_on_events",
+                desc=_("Show notifications on the system tray icon"),
+                bind="show_trayicon",
             ),
-        ]
-
-        for setting in settings:
-            self.add_setting(setting)
-
-
-class NotificationsGroup(GajimPreferencesGroup):
-    def __init__(self) -> None:
-        GajimPreferencesGroup.__init__(
-            self,
-            key="notifications",
-        )
-
-        settings = [
             Setting(
                 SettingKind.SWITCH,
-                _("Show Notifications"),
+                _("System Notifications"),
                 SettingType.CONFIG,
                 "show_notifications",
             ),
             Setting(
                 SettingKind.SWITCH,
-                _("Notifications When Away"),
+                _("System Notifications When Away"),
                 SettingType.CONFIG,
                 "show_notifications_away",
                 desc=_("Show notifications even if you are Away, Busy, etc."),
@@ -1097,6 +1056,7 @@ class AdvancedGroup(GajimPreferencesGroup):
                 _("Debug Logging"),
                 SettingType.VALUE,
                 app.get_debug_mode(),
+                desc=_("Restart Gajim to start debug logging"),
                 props={
                     "button-icon-name": "lucide-folder-symbolic",
                     "button-callback": self._on_open_debug_logs,
@@ -1146,7 +1106,7 @@ class AdvancedGroup(GajimPreferencesGroup):
 
     @staticmethod
     def _on_open_debug_logs(*args: Any) -> None:
-        open_directory(configpaths.get("DEBUG"))
+        open_file(configpaths.get("DEBUG"))
 
     @staticmethod
     def _on_advanced_config_editor(*args: Any) -> None:
@@ -1312,13 +1272,3 @@ class AutoExtendedAwayPage(GajimPreferencePage):
             title=_("Auto Not Available"),
             groups=[AutoExtendedAwayGroup],
         )
-
-
-class NotificationsPage(GajimPreferencePage):
-    def __init__(self) -> None:
-        GajimPreferencePage.__init__(
-            self,
-            key="notifications",
-            title=_("Notifications"),
-            groups=[NotificationsGroup],
-        )
